#!/usr/bin/env python
r"""
Flow for phonon-limited mobilities in Silicon
====================================================
"""
import sys
import os
import abipy.data as abidata
import abipy.abilab as abilab
import abipy.flowtk as flowtk
import abipy.core.abinit_units as abu
import numpy as np
def remove_values_from_list(the_list, val):
   return [value for value in the_list if value != val]

def get_param(filename):

    f=open(filename,"r")
    lines=f.readlines()
    f.close()
    
    xred=np.zeros((2,3))
    xcart=np.zeros((2,3))
    rprim=np.zeros((3,3))
    
    i=0
    end=False
    for line in lines:
      if line=="== END DATASET(S) ==============================================================\n":
        end=True
      new_line=remove_values_from_list(line.split(" "),"")
      if end:
        if new_line[0]=="xred":
            new_line=remove_values_from_list(lines[i].split(" "),"")
            xred[0,0]=float(new_line[1])
            xred[0,1]=float(new_line[2])
            xred[0,2]=float(new_line[3][:-1])      
            new_line=remove_values_from_list(lines[i+1].split(" "),"")
            xred[1,0]=float(new_line[0])
            xred[1,1]=float(new_line[1])
            xred[1,2]=float(new_line[2][:-1])      
            
        elif new_line[0]=="xcart":
            new_line=remove_values_from_list(lines[i].split(" "),"")
            xcart[0,0]=float(new_line[1])
            xcart[0,1]=float(new_line[2])
            xcart[0,2]=float(new_line[3][:-1])      
            new_line=remove_values_from_list(lines[i+1].split(" "),"")
            xcart[1,0]=float(new_line[0])
            xcart[1,1]=float(new_line[1])
            xcart[1,2]=float(new_line[2][:-1])  
               
        elif new_line[0]=="rprim":
            new_line=remove_values_from_list(lines[i].split(" "),"")
            rprim[0,0]=float(new_line[1])
            rprim[0,1]=float(new_line[2])
            rprim[0,2]=float(new_line[3][:-1])      
            new_line=remove_values_from_list(lines[i+1].split(" "),"")
            rprim[1,0]=float(new_line[0])
            rprim[1,1]=float(new_line[1])
            rprim[1,2]=float(new_line[2][:-1])              
            new_line=remove_values_from_list(lines[i+2].split(" "),"")
            rprim[2,0]=float(new_line[0])
            rprim[2,1]=float(new_line[1])
            rprim[2,2]=float(new_line[2][:-1])      
                             
      i=i+1
    return {"xred":xred,"xcart":xcart,"rprim":rprim}
def build_flow(options):

    # Working directory (default is the name of the script with '.py' removed and "run_" replaced by "flow_")
    if not options.workdir:
        options.workdir = os.path.basename(sys.argv[0]).replace(".py", "").replace("run_", "flow_")

    splt=options.workdir.split("/")
    cell_param=get_param("/gpfs/scratch/acad/elph/StrainedSi/Relaxation/%s/%s/run.abo"%(splt[-2],splt[-1]))
    
    # Initialize the flow
    print(splt)
    print(cell_param)
    flow = flowtk.Flow(workdir=options.workdir, manager=options.manager)

    # Initialize the tmesh, sigma_kerange, sigma_erange and dense meshes for the eph integrations
    tmesh = [100, 25, 25]
    sigma_kerange_n = [0, 0.50 * abu.eV_Ha] # 0.5 eV range for the WFK of electrons # first term for holes, second for electrons
    sigma_erange_n = [0, 0.2 * abu.eV_Ha] # 0.2 eV range for the EPH computation for electrons # first term for holes, second for electrons
    
    sigma_kerange_p = [0.50 * abu.eV_Ha,0] # 0.5 eV range for the WFK of electrons # first term for holes, second for electrons
    sigma_erange_p = [0.2 * abu.eV_Ha, 0] # 0.2 eV range for the EPH computation for electrons # first term for holes, second for electrons    
    eph_doping=-1e15 # for doping effect in eph_task minus for electrons (n-doped) and positive for holes (p-doped)
    occopt=3 #  occopt to 3 to correctly compute the location of the Fermi level using the Fermi-Dirac occupation function as we are dealing with the physical temperature 
    dense_meshes = [[150, 150, 150]]
    ibte_abs_tol=1e-8
    ibte_niter=200
    
    kptbounds=  [[0.5,0.5,0], [0,0,0],[0,0.5,0.5],[0,0,0],[0.5,0,0.5],[0.625,0.25,0.625],[0.5,0.25,0.75],[0.375,0.375,0.75],[0,0,0],[0.5,0.5,0.5],[0.5,0.25,0.75],[0,0.5,0.5]]
    # X-G-X-G-X-U-W-K-G-L-W-X
    
    
    # Initialize the structure and pseudos
    structure =  dict(
                        ntypat=1,         
                        natom=2,
                        typat=[1, 1],
                        znucl=14,
                        acell=[10.34166194583063, 10.34166194583063,10.34166194583063 ], # 10.334913640
                        rprim=cell_param["rprim"],
                        xred=cell_param["xred"]
                        )
  
    pseudos = "Si_woNLCC.psp8"

    # Ground-state computation for the phonons and Q* (nband = nocc)
    scf_input = abilab.AbinitInput(structure, pseudos=pseudos)

    scf_input.set_vars(
        nband=4,
        ecut=20,
        ngkpt=[18, 18, 18],
        shiftk=[0, 0, 0],
        tolvrs=1.0e-12,
        diemac=12.0,
        prtden=1,
        nstep=100,
    )

    # Initialize the first work and add the ground-state task
    work_gs = flowtk.Work()
    work_gs.register_scf_task(scf_input)
    
    # Ground-state computation for the WFK (more empty bands)
    scf_input = abilab.AbinitInput(structure, pseudos=pseudos)

    scf_input.set_vars(
        nband=10,
        nbdbuf=2,
        ecut=20,
        ngkpt=[18, 18, 18],
        shiftk=[0, 0, 0],
        tolvrs=1.0e-12,
        diemac=12.0,
        prtden=1,
        nstep=100,
    )

    # Add the second ground-state task to the first work
    work_gs.register_scf_task(scf_input)

    # Band structure calculation to make sure everything is OK
    # Also allows to compare the results obtained with abitk to
    # check the SKW interpolation works as needed
    bs_input = scf_input.make_ebands_input(tolwfr=1e-12, ndivsm=10, nb_extra=4)
    bs_input.set_vars(nstep=100, nbdbuf=2,kptbounds=kptbounds)
    work_gs.register_nscf_task(bs_input, deps={work_gs[1]: "DEN"})

    # NSCF input for the WFK needed to interpolate with kerange
    nscf_input = abilab.AbinitInput(structure, pseudos)
    nscf_input.set_vars(
        ecut=20,
        nband=10,
        nbdbuf=2,
        iscf=-2,
        tolwfr=1e-20,
        prtwf=1,
        ngkpt=[36, 36, 36], # Should be dense enough so that the kerange interpolation works
        shiftk=[0.0, 0.0, 0.0],
        nstep=100,
    )

    work_gs.register_nscf_task(nscf_input, deps={work_gs[1]: "DEN"})
    flow.register_work(work_gs)

    # Add the phonon work to the flow
    ddb_ngqpt = [9,9, 9]
    work_ph = flowtk.works.PhononWork.from_scf_task(work_gs[0], qpoints=ddb_ngqpt,
                                              is_ngqpt=True, with_becs=True, with_quad=True)
    flow.register_work(work_ph)

    # We loop over the dense meshes
    for i, sigma_ngkpt in enumerate(dense_meshes):
        # Use the kerange trick to generate a WFK file
        multi_n = nscf_input.make_wfk_kerange_inputs(sigma_kerange=sigma_kerange_n,
                                                   sigma_ngkpt=sigma_ngkpt)
        kerange_input_n, wfk_input_n = multi_n.split_datasets()

        work_eph_n = flowtk.Work()
        work_eph_n.register_kerange_task(kerange_input_n, deps={work_gs[3]: "WFK"})
        work_eph_n.register_nscf_task(wfk_input_n, deps={work_gs[1]: "DEN", 
                                                     work_eph_n[0]: "KERANGE.nc"})

        # Generate the input file for the transport calculation.
        # Use ibte_prep = 1 to activate the iterative BTE.
        eph_input_n = wfk_input_n.make_eph_transport_input(ddb_ngqpt=ddb_ngqpt,
                                                       sigma_erange=sigma_erange_n,
                                                       tmesh=tmesh,
                                                       eph_ngqpt_fine=sigma_ngkpt,
                                                       ibte_prep=1,ibte_niter=ibte_niter,ibte_abs_tol=ibte_abs_tol)

        # We compute the phonon dispersion in the EPH code to be able to check they are ok.
        if i == 0:
            eph_input_n.set_qpath(20,qptbounds=kptbounds)

        work_eph_n.register_eph_task(eph_input_n, deps={work_eph_n[1]: "WFK", 
                                                    work_ph: ["DDB", "DVDB"]})

        flow.register_work(work_eph_n)

    for i, sigma_ngkpt in enumerate(dense_meshes):
        # Use the kerange trick to generate a WFK file
        multi_p = nscf_input.make_wfk_kerange_inputs(sigma_kerange=sigma_kerange_p,
                                                   sigma_ngkpt=sigma_ngkpt)
        kerange_input_p, wfk_input_p = multi_p.split_datasets()

        work_eph_p = flowtk.Work()
        work_eph_p.register_kerange_task(kerange_input_p, deps={work_gs[3]: "WFK"})
        work_eph_p.register_nscf_task(wfk_input_p, deps={work_gs[1]: "DEN", 
                                                     work_eph_p[0]: "KERANGE.nc"})

        # Generate the input file for the transport calculation.
        # Use ibte_prep = 1 to activate the iterative BTE.
        eph_input_p = wfk_input_p.make_eph_transport_input(ddb_ngqpt=ddb_ngqpt,
                                                       sigma_erange=sigma_erange_p,
                                                       tmesh=tmesh,
                                                       eph_ngqpt_fine=sigma_ngkpt,
                                                       ibte_prep=1,ibte_niter=ibte_niter,ibte_abs_tol=ibte_abs_tol)

        # We compute the phonon dispersion in the EPH code to be able to check they are ok.
        if i == 0:
            eph_input_p.set_qpath(20,qptbounds=kptbounds)

        work_eph_p.register_eph_task(eph_input_p, deps={work_eph_p[1]: "WFK", 
                                                    work_ph: ["DDB", "DVDB"]})

        flow.register_work(work_eph_p)
        
    flow.allocate(use_smartio=True)

    return flow

@flowtk.flow_main
def main(options):
    """
    This is our main function that will be invoked by the script.
    flow_main is a decorator implementing the command line interface.
    Command line args are stored in `options`.
    """

    return build_flow(options)


if __name__ == "__main__":
    sys.exit(main())
    
#flow=build_flow()
#flow.make_scheduler().start()
