#!/usr/bin/env python
r"""
Flow for phonon-limited mobilities in Silicon
====================================================
"""
import sys
import os
import abipy.data as abidata
import abipy.abilab as abilab
import abipy.flowtk as flowtk
import abipy.core.abinit_units as abu

def build_flow(options):

    # Working directory (default is the name of the script with '.py' removed and "run_" replaced by "flow_")
    if not options.workdir:
        options.workdir = os.path.basename(sys.argv[0]).replace(".py", "").replace("run_", "flow_")

    # Initialize the flow
    flow = flowtk.Flow(workdir=options.workdir, manager=options.manager)

    # Initialize the structure and pseudos
    structure = abilab.Structure.from_abivars(acell=3*[1.0],
                                              rprim=[0.00000000, 5.08388087, 5.08388087,
                                                     5.08388087, 0.00000000, 5.08388087,
                                                     5.08388087, 5.08388087, 0.0000000],
                                              ntypat=1,
                                              typat=[1, 1],
                                              znucl=[14],
                                              xred=[0.0, 0.0, 0.0,
                                                    0.25, 0.25, 0.25])


    # Relaxation of the unit cell (atoms only)
    relax_input = abilab.AbinitInput(structure, pseudos="/gpfs/scratch/acad/elph/StrainedSi/Si_woNLCC.psp8")

    relax_input.set_vars(
        nband=8,
        ecut=20,
        kptopt=1,
        ngkpt=[18, 18, 18],
        shiftk=[0, 0, 0],
        tolvrs=1e-12,
        diemac=12.0,
        optcell=0,
        ionmov=2,
        ntime=100,
        nstep=100)

    # Initialize the relax work and add the relaxation task
    work_relax = flowtk.Work()
    work_relax.register_relax_task(relax_input)
    flow.register_work(work_relax)
    
    return flow

@flowtk.flow_main
def main(options):
    """
    This is our main function that will be invoked by the script.
    flow_main is a decorator implementing the command line interface.
    Command line args are stored in `options`.
    """

    return build_flow(options)


if __name__ == "__main__":
    sys.exit(main())
